@@ -21,11 +21,11 @@ from astropy.io.fits.diff import FITSDiff
 from astropy.io.fits.file import GZIP_MAGIC, _File
 from astropy.io.tests import safeio
 from astropy.utils import data
-
-# NOTE: Python can be built without bz2.
-from astropy.utils.compat.optional_deps import HAS_BZ2
+from astropy.utils.compat.optional_deps import (
+    HAS_BZ2,  # NOTE: Python can be built without bz2
+)
 from astropy.utils.data import conf
-from astropy.utils.exceptions import AstropyUserWarning
+from astropy.utils.exceptions import AstropyDeprecationWarning, AstropyUserWarning
 from astropy.utils.misc import _NOT_OVERWRITING_MSG_MATCH
 
 from .conftest import FitsTestCase
@@ -158,7 +158,7 @@ class TestCore(FitsTestCase):
         assert header.comments["BITPIX"] == ""
 
     def test_set_card_value(self):
-        """Similar to test_update_header_card(), but tests the the
+        """Similar to test_update_header_card(), but tests the
         `header['FOO'] = 'bar'` method of updating card values.
         """
 
@@ -280,9 +280,10 @@ class TestCore(FitsTestCase):
         # silentfix+warn should be quiet about the fixed HDU and only warn
         # about the unfixable one
         hdu = make_invalid_hdu()
-        with pytest.warns(AstropyUserWarning, match="Illegal keyword name") as w:
+        with pytest.warns(fits.verify.VerifyWarning) as w:
             hdu.verify("silentfix+warn")
         assert len(w) == 4
+        assert "Illegal keyword name 'P.I.'" in str(w.list[2].message)
 
         # silentfix+exception should only mention the unfixable error in the
         # exception
@@ -294,9 +295,10 @@ class TestCore(FitsTestCase):
         # fix+ignore is not too useful, but it should warn about the fixed
         # problems while saying nothing about the unfixable problems
         hdu = make_invalid_hdu()
-        with pytest.warns(AstropyUserWarning, match="not upper case") as w:
+        with pytest.warns(fits.verify.VerifyWarning) as w:
             hdu.verify("fix+ignore")
         assert len(w) == 4
+        assert "not upper case" not in str(w.list[0].message)
 
         # fix+warn
         hdu = make_invalid_hdu()
@@ -506,7 +508,7 @@ class TestCore(FitsTestCase):
         h1.header["EXTVER"] = 3
         assert h1.ver == 3
         del h1.header["EXTVER"]
-        h1.ver == 1
+        assert h1.ver == 1
 
         h1.level = 2
         assert h1.header.get("EXTLEVEL") == 2
@@ -602,6 +604,10 @@ class TestConvenienceFunctions(FitsTestCase):
             assert len(hdul) == 1
             assert (data == hdul[0].data).all()
 
+    def test_writeto_stdout(self):
+        # see https://github.com/astropy/astropy/issues/3427
+        fits.writeto(sys.stdout, data=np.array([1, 2]))
+
 
 class TestFileFunctions(FitsTestCase):
     """
@@ -709,9 +715,15 @@ class TestFileFunctions(FitsTestCase):
 
     def test_open_gzipped(self):
         gzip_file = self._make_gzip_file()
+
         with fits.open(gzip_file) as fits_handle:
             assert fits_handle._file.compression == "gzip"
             assert len(fits_handle) == 5
+
+        with fits.open(gzip_file, decompress_in_memory=True) as fits_handle:
+            assert fits_handle._file.compression == "gzip"
+            assert len(fits_handle) == 5
+
         with fits.open(gzip.GzipFile(gzip_file)) as fits_handle:
             assert fits_handle._file.compression == "gzip"
             assert len(fits_handle) == 5
@@ -753,12 +765,12 @@ class TestFileFunctions(FitsTestCase):
         """Test updating a GZipped FITS file"""
 
         with fits.open(self._make_gzip_file("update.gz"), mode="update") as fits_handle:
-            hdu = fits.ImageHDU(data=[x for x in range(100)])
+            hdu = fits.ImageHDU(data=list(range(100)))
             fits_handle.append(hdu)
 
         with fits.open(self.temp("update.gz")) as new_handle:
             assert len(new_handle) == 6
-            assert (new_handle[-1].data == [x for x in range(100)]).all()
+            assert (new_handle[-1].data == list(range(100))).all()
 
     def test_fits_append_mode_gzip(self):
         """Make sure that attempting to open an existing GZipped FITS file in
@@ -771,10 +783,15 @@ class TestFileFunctions(FitsTestCase):
     @pytest.mark.skipif(not HAS_BZ2, reason="Python built without bz2 module")
     def test_open_bzipped(self):
         bzip_file = self._make_bzip2_file()
+
         with fits.open(bzip_file) as fits_handle:
             assert fits_handle._file.compression == "bzip2"
             assert len(fits_handle) == 5
 
+        with fits.open(bzip_file, decompress_in_memory=True) as fits_handle:
+            assert fits_handle._file.compression == "bzip2"
+            assert len(fits_handle) == 5
+
         with fits.open(bz2.BZ2File(bzip_file)) as fits_handle:
             assert fits_handle._file.compression == "bzip2"
             assert len(fits_handle) == 5
@@ -818,9 +835,15 @@ class TestFileFunctions(FitsTestCase):
 
     def test_open_zipped(self):
         zip_file = self._make_zip_file()
+
         with fits.open(zip_file) as fits_handle:
             assert fits_handle._file.compression == "zip"
             assert len(fits_handle) == 5
+
+        with fits.open(zip_file, decompress_in_memory=True) as fits_handle:
+            assert fits_handle._file.compression == "zip"
+            assert len(fits_handle) == 5
+
         with fits.open(zipfile.ZipFile(zip_file)) as fits_handle:
             assert fits_handle._file.compression == "zip"
             assert len(fits_handle) == 5
@@ -1333,7 +1356,7 @@ class TestFileFunctions(FitsTestCase):
                 (2, "Canopus", -0.73, "F0Ib"),
                 (3, "Rigil Kent", -0.1, "G2V"),
             ],
-            formats="int16,a20,float32,a10",
+            formats="int16,S20,float32,S10",
             names="order,name,mag,Sp",
         )
 
@@ -1355,6 +1378,11 @@ class TestFileFunctions(FitsTestCase):
         hdu_img_2880.writeto(fh)
         fh.close()
 
+    def test_HDUList_writeto_stdout(self):
+        # see https://github.com/astropy/astropy/issues/3427
+        hdul = fits.HDUList([fits.PrimaryHDU()])
+        hdul.writeto(sys.stdout)
+
 
 class TestStreamingFunctions(FitsTestCase):
     """Test functionality of the StreamingHDU class."""
@@ -1429,13 +1457,11 @@ class TestStreamingFunctions(FitsTestCase):
         hdul = fits.HDUList([phdu, ihdu])
         filename = self.temp("temp.fits")
 
-        pytest.raises(
-            fits.VerifyError, hdul.writeto, filename, output_verify="exception"
-        )
-        with pytest.warns(
-            fits.verify.VerifyWarning, match=r"Verification reported errors"
-        ):
+        with pytest.raises(fits.VerifyError):
+            hdul.writeto(filename, output_verify="exception")
+        with pytest.warns(fits.verify.VerifyWarning) as w:
             hdul.writeto(filename, output_verify="fix")
+        assert "Verification reported errors" in str(w[0].message)
         with fits.open(filename):
             assert hdul[1].name == "12345678"
             assert hdul[1].header["EXTNAME"] == "12345678"
@@ -1466,3 +1492,12 @@ class TestStreamingFunctions(FitsTestCase):
         # See https://github.com/astropy/astropy/issues/3766
         with fits.open(pth, memmap=True, do_not_scale_image_data=True) as hdul:
             hdul[0].data  # Just make sure it doesn't crash
+
+
+def test_deprecated_hdu_classes():
+    from astropy.io.fits.hdu.base import _ExtensionHDU, _NonstandardExtHDU
+
+    with pytest.warns(AstropyDeprecationWarning):
+        _ExtensionHDU()
+    with pytest.warns(AstropyDeprecationWarning):
+        _NonstandardExtHDU()
