@@ -90,7 +90,6 @@ SUBCLASS_SAFE_FUNCTIONS |= {
     np.nansum, np.nancumsum, np.nanstd, np.nanvar,
     np.nanprod, np.nancumprod,
     np.einsum_path, np.linspace,
-    np.trapz,  # deprecated in not NUMPY_LT_2_0
     np.sort, np.partition, np.meshgrid,
     np.common_type, np.result_type, np.can_cast, np.min_scalar_type,
     np.iscomplexobj, np.isrealobj,
@@ -98,18 +97,36 @@ SUBCLASS_SAFE_FUNCTIONS |= {
     np.apply_along_axis, np.take_along_axis, np.put_along_axis,
     np.linalg.cond, np.linalg.multi_dot,
 }  # fmt: skip
-SUBCLASS_SAFE_FUNCTIONS |= {  # Deprecated
-    np.product, np.cumproduct,  # noqa: NPY003
-}  # fmt: skip
 
 SUBCLASS_SAFE_FUNCTIONS |= {np.median}
 
 if NUMPY_LT_2_0:
-    # functions removed in numpy 2.0; alias for np.round in NUMPY_LT_1_25
-    SUBCLASS_SAFE_FUNCTIONS |= {np.msort, np.round_}  # noqa: NPY003
+    # functions (re)moved in numpy 2.0; alias for np.round in NUMPY_LT_1_25
+    SUBCLASS_SAFE_FUNCTIONS |= {
+        np.msort,
+        np.round_,  # noqa: NPY003
+        np.trapz,
+        np.product,  # noqa: NPY003
+        np.cumproduct,  # noqa: NPY003
+    }
 else:
     # Array-API compatible versions (matrix axes always at end).
-    SUBCLASS_SAFE_FUNCTIONS |= {np.linalg.diagonal, np.linalg.trace}
+    SUBCLASS_SAFE_FUNCTIONS |= {
+        np.matrix_transpose, np.linalg.matrix_transpose,
+        np.linalg.diagonal, np.linalg.trace,
+        np.linalg.matrix_norm, np.linalg.vector_norm, np.linalg.vecdot,
+    }  # fmt: skip
+
+    # these work out of the box (and are tested), because they
+    # delegate to other, already wrapped functions from the np namespace
+    SUBCLASS_SAFE_FUNCTIONS |= {
+        np.linalg.cross, np.linalg.svdvals, np.linalg.tensordot, np.linalg.matmul,
+        np.unique_all, np.unique_counts, np.unique_inverse, np.unique_values,
+        np.astype,
+    }  # fmt: skip
+
+    # trapz was renamed to trapezoid
+    SUBCLASS_SAFE_FUNCTIONS |= {np.trapezoid}
 
 # Implemented as methods on Quantity:
 # np.ediff1d is from setops, but we support it anyway; the others
@@ -122,9 +139,11 @@ UNSUPPORTED_FUNCTIONS |= {
     np.busday_count, np.busday_offset, np.datetime_as_string,
     np.is_busday, np.all, np.any,
 }  # fmt: skip
-UNSUPPORTED_FUNCTIONS |= {  # Deprecated
-    np.sometrue, np.alltrue,  # noqa: NPY003
-}  # fmt: skip
+
+if NUMPY_LT_2_0:
+    UNSUPPORTED_FUNCTIONS |= {  # removed in numpy 2.0
+        np.sometrue, np.alltrue,  # noqa: NPY003
+    }  # fmt: skip
 
 # Could be supported if we had a natural logarithm unit.
 UNSUPPORTED_FUNCTIONS |= {np.linalg.slogdet}
@@ -904,14 +923,24 @@ def interp(x, xp, fp, *args, **kwargs):
 
 @function_helper
 def unique(
-    ar, return_index=False, return_inverse=False, return_counts=False, axis=None
+    ar,
+    return_index=False,
+    return_inverse=False,
+    return_counts=False,
+    axis=None,
+    **kwargs,
 ):
     unit = ar.unit
     n_index = sum(bool(i) for i in (return_index, return_inverse, return_counts))
     if n_index:
         unit = [unit] + n_index * [None]
 
-    return (ar.value, return_index, return_inverse, return_counts, axis), {}, unit, None
+    return (
+        (ar.value, return_index, return_inverse, return_counts, axis),
+        kwargs,
+        unit,
+        None,
+    )
 
 
 @function_helper
@@ -1066,11 +1095,29 @@ def inv(a, *args, **kwargs):
     return (a.view(np.ndarray),) + args, kwargs, 1 / a.unit, None
 
 
-@function_helper(module=np.linalg)
-def pinv(a, rcond=1e-15, *args, **kwargs):
-    rcond = _interpret_tol(rcond, a.unit)
+if NUMPY_LT_2_0:
 
-    return (a.view(np.ndarray), rcond) + args, kwargs, 1 / a.unit, None
+    @function_helper(module=np.linalg)
+    def pinv(a, rcond=1e-15, *args, **kwargs):
+        rcond = _interpret_tol(rcond, a.unit)
+
+        return (a.view(np.ndarray), rcond) + args, kwargs, 1 / a.unit, None
+
+else:
+
+    @function_helper(module=np.linalg)
+    def pinv(a, rcond=None, hermitian=False, *, rtol=np._NoValue):
+        if rcond is not None:
+            rcond = _interpret_tol(rcond, a.unit)
+        if rtol is not np._NoValue and rtol is not None:
+            rtol = _interpret_tol(rtol, a.unit)
+
+        return (
+            (a.view(np.ndarray),),
+            dict(rcond=rcond, hermitian=hermitian, rtol=rtol),
+            1 / a.unit,
+            None,
+        )
 
 
 @function_helper(module=np.linalg)
@@ -1091,7 +1138,7 @@ def solve(a, b, *args, **kwargs):
 
 
 @function_helper(module=np.linalg)
-def lstsq(a, b, rcond="warn"):
+def lstsq(a, b, rcond="warn" if NUMPY_LT_2_0 else None):
     a, b = _as_quantities(a, b)
 
     if rcond not in (None, "warn", -1):
@@ -1121,9 +1168,17 @@ def matrix_power(a, n):
     return (a.value, n), {}, a.unit**n, None
 
 
-@function_helper(module=np.linalg)
-def cholesky(a):
-    return (a.value,), {}, a.unit**0.5, None
+if NUMPY_LT_2_0:
+
+    @function_helper(module=np.linalg)
+    def cholesky(a):
+        return (a.value,), {}, a.unit**0.5, None
+
+else:
+
+    @function_helper(module=np.linalg)
+    def cholesky(a, /, *, upper=False):
+        return (a.value,), {"upper": upper}, a.unit**0.5, None
 
 
 @function_helper(module=np.linalg)
@@ -1147,6 +1202,17 @@ def eig(a, *args, **kwargs):
     return (a.value,) + args, kwargs, (a.unit, dimensionless_unscaled), None
 
 
+if not NUMPY_LT_2_0:
+    # these functions were added in numpy 2.0
+
+    @function_helper(module=np.linalg)
+    def outer(a, b, /):
+        # maybe this one can be marked as subclass-safe in the near future ?
+        # see https://github.com/numpy/numpy/pull/25101#discussion_r1419879122
+        a, b = _as_quantities(a, b)
+        return (a.view(np.ndarray), b.view(np.ndarray)), {}, a.unit * b.unit, None
+
+
 # ======================= np.lib.recfunctions =======================
 
 
